@@ -1,9 +1,9 @@
 /******************************************************************************
- * $Id: tif_overview.c,v 1.3 2000/04/18 22:48:31 warmerda Exp $
+ * tif_overview.c,v 1.9 2005/05/25 09:03:16 dron Exp
  *
  * Project:  TIFF Overview Builder
  * Purpose:  Library function for building overviews in a TIFF file.
- * Author:   Frank Warmerdam, warmerda@home.com
+ * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
  * Notes:
  *  o Currently only images with bits_per_sample of a multiple of eight
@@ -43,27 +43,16 @@
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ******************************************************************************
- *
- * $Log: tif_overview.c,v $
- * Revision 1.3  2000/04/18 22:48:31  warmerda
- * Added support for averaging resampling
- *
- * Revision 1.2  2000/01/28 15:36:38  warmerda
- * pass TIFF handle instead of filename to overview builder
- *
- * Revision 1.1  2000/01/28 15:04:03  warmerda
- * New
- *
  */
 
 #include <stdio.h>
 #include <assert.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "tiffio.h"
 #include "tif_ovrcache.h"
 
-
 #ifndef FALSE
 #  define FALSE 0
 #  define TRUE 1
@@ -84,11 +73,10 @@ void TIFFBuildOverviews( TIFF *, int, int *, int, const char *,
 /*      Returns offset of newly created overview directory, but the     */
 /*      current directory is reset to be the one in used when this      */
 /*      function is called.                                             */
-/********************************* ***************************************/
+/************************************************************************/
 
-static
 uint32 TIFF_WriteOverview( TIFF *hTIFF, int nXSize, int nYSize,
-                           int nBitsPerPixel, int nSamples, 
+                           int nBitsPerPixel, int nPlanarConfig, int nSamples, 
                            int nBlockXSize, int nBlockYSize,
                            int bTiled, int nCompressFlag, int nPhotometric,
                            int nSampleFormat,
@@ -113,7 +101,7 @@ uint32 TIFF_WriteOverview( TIFF *hTIFF, int nXSize, int nYSize,
     if( nSamples == 1 )
         TIFFSetField( hTIFF, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG );
     else
-        TIFFSetField( hTIFF, TIFFTAG_PLANARCONFIG, PLANARCONFIG_SEPARATE );
+        TIFFSetField( hTIFF, TIFFTAG_PLANARCONFIG, nPlanarConfig );
 
     TIFFSetField( hTIFF, TIFFTAG_BITSPERSAMPLE, nBitsPerPixel );
     TIFFSetField( hTIFF, TIFFTAG_SAMPLESPERPIXEL, nSamples );
@@ -142,9 +130,11 @@ uint32 TIFF_WriteOverview( TIFF *hTIFF, int nXSize, int nYSize,
 /* -------------------------------------------------------------------- */
 /*      Write directory, and return byte offset.                        */
 /* -------------------------------------------------------------------- */
-    TIFFWriteCheck( hTIFF, bTiled, "TIFFBuildOverviews" );
+    if( TIFFWriteCheck( hTIFF, bTiled, "TIFFBuildOverviews" ) == 0 )
+        return 0;
+
     TIFFWriteDirectory( hTIFF );
-    TIFFSetDirectory( hTIFF, TIFFNumberOfDirectories(hTIFF)-1 );
+    TIFFSetDirectory( hTIFF, (tdir_t) (TIFFNumberOfDirectories(hTIFF)-1) );
 
     nOffset = TIFFCurrentDirOffset( hTIFF );
 
@@ -281,8 +271,8 @@ void TIFF_DownSample( unsigned char *pabySrcTile,
         if( j + nTYOff >= nOBlockYSize )
             break;
             
-        pabyDst = pabyOTile
-            + ((j+nTYOff)*nOBlockXSize + nTXOff) * nPixelBytes;
+        pabyDst = pabyOTile + ((j+nTYOff)*nOBlockXSize + nTXOff)
+            * nPixelBytes * nPixelGroupBytes;
 
 /* -------------------------------------------------------------------- */
 /*      Handler nearest resampling ... we don't even care about the     */
@@ -304,10 +294,9 @@ void TIFF_DownSample( unsigned char *pabySrcTile,
                  */
 
                 for( k = 0; k < nPixelBytes; k++ )
-                {
-                    *(pabyDst++) = pabySrc[k];
-                }
-            
+                    pabyDst[k] = pabySrc[k];
+
+                pabyDst += nPixelBytes * nPixelGroupBytes;
                 pabySrc += nOMult * nPixelGroupBytes;
             }
         }
@@ -389,14 +378,15 @@ void TIFF_ProcessFullResBlock( TIFF *hTIFF, int nPlanarConfig,
             {
                 TIFFReadEncodedTile( hTIFF,
                                      TIFFComputeTile(hTIFF, nSXOff, nSYOff,
-                                                     0, iSample ),
+                                                     0, (tsample_t)iSample ),
                                      pabySrcTile,
                                      TIFFTileSize(hTIFF));
             }
             else
             {
                 TIFFReadEncodedStrip( hTIFF,
-                                      TIFFComputeStrip(hTIFF, nSYOff, iSample),
+                                      TIFFComputeStrip(hTIFF, nSYOff,
+                                                       (tsample_t) iSample),
                                       pabySrcTile,
                                       TIFFStripSize(hTIFF) );
             }
@@ -540,14 +530,15 @@ void TIFFBuildOverviews( TIFF *hTIFF, int nOverviews, int * panOvList,
                       &panRedMap, &panGreenMap, &panBlueMap ) )
     {
         uint16		*panRed2, *panGreen2, *panBlue2;
+        int             nColorCount = 1 << nBitsPerPixel;
 
-        panRed2 = (uint16 *) _TIFFmalloc(2*256);
-        panGreen2 = (uint16 *) _TIFFmalloc(2*256);
-        panBlue2 = (uint16 *) _TIFFmalloc(2*256);
+        panRed2 = (uint16 *) _TIFFmalloc(2*nColorCount);
+        panGreen2 = (uint16 *) _TIFFmalloc(2*nColorCount);
+        panBlue2 = (uint16 *) _TIFFmalloc(2*nColorCount);
 
-        memcpy( panRed2, panRedMap, 512 );
-        memcpy( panGreen2, panGreenMap, 512 );
-        memcpy( panBlue2, panBlueMap, 512 );
+        memcpy( panRed2, panRedMap, 2 * nColorCount );
+        memcpy( panGreen2, panGreenMap, 2 * nColorCount );
+        memcpy( panBlue2, panBlueMap, 2 * nColorCount );
 
         panRedMap = panRed2;
         panGreenMap = panGreen2;
@@ -584,8 +575,8 @@ void TIFFBuildOverviews( TIFF *hTIFF, int nOverviews, int * panOvList,
         }
 
         nDirOffset = TIFF_WriteOverview( hTIFF, nOXSize, nOYSize,
-                                         nBitsPerPixel, nSamples,
-                                         nOBlockXSize, nOBlockYSize,
+                                         nBitsPerPixel, nPlanarConfig,
+                                         nSamples, nOBlockXSize, nOBlockYSize,
                                          bTiled, nCompressFlag, nPhotometric,
                                          nSampleFormat,
                                          panRedMap, panGreenMap, panBlueMap,
